-- --------------------------------------------------
--  Main Multiplexer Description
-- --------------------------------------------------
library IEEE;
use IEEE.std_logic_1164.all;

entity LPTAD is
	port (
			stat            : in  std_logic_vector (3 downto 0);
			
			lpt_datastrobe  : in std_logic;
			lpt_data        : inout std_logic_vector (7 downto 0);
			lpt_ack         : out std_logic;
			lpt_busy        : out std_logic;
			lpt_paperout    : out std_logic;
			lpt_select      : out std_logic;
			lpt_autofeed    : in std_logic;
			lpt_error       : out std_logic;
			lpt_init        : in std_logic;
			lpt_selectinput : in std_logic;
			
			ext_1o : out std_logic;  -- TRST / RST
			ext_2o : out std_logic;  -- TDI  / MOSI
			ext_3o : out std_logic;  -- RTCK / XTAL
			ext_4o : out std_logic;  -- TCK  / SCK
			ext_5i : in  std_logic;  -- TDO  / MISO
			ext_6o : out std_logic;  -- TMS  / -
			ext_7o : out std_logic;  -- RST  / -
			
			ext_1x1o : out std_logic;  -- SCL  / -
			ext_1x2o : out std_logic;  -- SDA  / -
			ext_1x3i : in  std_logic;  -- SDA  / -
			ext_1x4i : in  std_logic   -- SCL  / -
			
			);
end LPTAD;
architecture MAIN of LPTAD is
	signal in_lpt_data   : std_logic_vector (7 downto 0);
	signal out_lpt_data  : std_logic_vector (7 downto 0);
	signal lpt_oe        : std_logic_vector (7 downto 0);
	component bidir is 
		port (
				oe         : in std_logic;
				ext_port   : inout std_logic;
				in_data    : in std_logic;
				out_data   : out std_logic
			);
	end component;
begin
	-- @todo    case  ,   
	--          .   
	--            .
	--           case  
	--         , , ,   
	--       .
	-- @todo    -     
	--       ,     3 ,   
	--          .  ,     
	--        , , ,   .
	process (stat)
	begin
		case (stat) is
			-- JTAG Wiggler 
			-- @todo RTCK signal
			when "0001" =>
				lpt_oe <= "00000000";
				lpt_busy <= ext_5i;           -- TDO
				--
				--ext_1o <= not in_lpt_data(0); -- RST
				--ext_7o <= not in_lpt_data(0); -- TRST
				--  ,   RESET ,  
				--      .
				if (in_lpt_data(0) = '1') then 
					ext_1o <= '0'; -- RST
					ext_7o <= '0'; -- TRST
				else
					ext_1o <= 'Z'; -- RST
					ext_7o <= 'Z'; -- TRST
				end if;
				ext_6o <= in_lpt_data(1);     -- TMS
				ext_4o <= in_lpt_data(2);     -- TCK
				ext_2o <= in_lpt_data(3);     -- TDI
				                              -- RTCK (???)
				
			-- STK200
			when "0010" => 
				lpt_oe <= "00000000";
				if (in_lpt_data(2) = '1') then 
					lpt_ack <= 'Z';      -- MISO
				else
					lpt_ack <= ext_5i;   -- MISO
				end if;
				if (in_lpt_data(3) = '1') then 
					ext_4o       <= 'Z'; -- SCK
					ext_2o       <= 'Z'; -- MOSI
					ext_3o       <= 'Z'; -- XTAL
					ext_1o       <= 'Z'; -- RST
				else
					ext_4o       <= in_lpt_data(4); -- SCK
					ext_2o       <= in_lpt_data(5); -- MOSI
					ext_3o       <= in_lpt_data(6); -- XTAL
					ext_1o       <= in_lpt_data(7); -- RST
				end if;
				
			-- STK200 (old, without HC244)
			--when "0010" => 
			--	lpt_paperout <= in_lpt_data(0); -- jmp
			--	lpt_busy     <= in_lpt_data(1); -- jmp
			--	-- in_lpt_data(2) ignored
			--	-- in_lpt_data(3) ignored
			--	lpt_ack      <= ext_5i;         -- MISO
			--	ext_4o       <= in_lpt_data(4); -- SCK
			--	ext_2o       <= in_lpt_data(5); -- MOSI
			--	ext_3o       <= in_lpt_data(6); -- XTAL
			--	ext_1o       <= in_lpt_data(7); -- RST
				
			-- ALTERA ByteBlaster 
			-- @todo
			--when "0010" => 
			--	lpt_oe <= "00000000";
				
			-- XILINX JTAG 
			-- now only JTAG mode
			when "0011" => 
				lpt_oe       <= "00000000";
				ext_2o       <= in_lpt_data(0); -- TDI
				ext_4o       <= in_lpt_data(3); -- TCK
				lpt_select   <= ext_5i;         -- TDO
				ext_6o       <= in_lpt_data(4); -- TMS
				lpt_error    <= '1';            -- Power On
				lpt_busy     <= in_lpt_data(6); -- jmp
				lpt_paperout <= in_lpt_data(6); -- jmp
				
			-- RFU / XDS510PP (TMS320 JTAG)
			--when "0101" => 
			--	lpt_oe <= "00000000";
				
			-- MSP430 JTAG 
			-- @todo
			--when "0110" => 
			--	lpt_oe <= "00000000";
				
			-- PIC ICSP
			-- @todo 
			--when "0111" => 
			--	lpt_oe <= "00000000";
				
			-- SPI EEPROM MEMORY
			-- @todo
			--when "1000" => 
			--	lpt_oe <= "00000000";
				
			-- I2C EEPROM MEMORY 
			when "1001" => 
				lpt_oe <= "00000000";
				ext_1o   <= '0';
				ext_2o   <= '0';
				ext_3o   <= '0';
				ext_4o   <= '0';
				ext_6o   <= '0';
				ext_7o   <= '0';
				ext_1x1o <= in_lpt_data(0);   -- SCL
				ext_1x2o <= in_lpt_data(1);   -- SDA
				lpt_select   <= ext_1x4i;     -- SCL
				lpt_paperout <= ext_1x3i;     -- SDA
				
			when others =>
				lpt_oe   <= "00000000";
				ext_1o   <= '0';
				ext_2o   <= '0';
				ext_3o   <= '0';
				ext_4o   <= '0';
				ext_6o   <= '0';
				ext_7o   <= '0';
				ext_1x1o <= '0';
				ext_1x2o <= '0';
		end case;
	end process;
	
	-- @todo     LPT-    .
	--          bidir   ,  
	--         .     ,   
	--           (      
	--         ).
	U0: bidir port map (lpt_oe(0), lpt_data(0), in_lpt_data(0), out_lpt_data(0));
	U1: bidir port map (lpt_oe(1), lpt_data(1), in_lpt_data(1), out_lpt_data(1));
	U2: bidir port map (lpt_oe(2), lpt_data(2), in_lpt_data(2), out_lpt_data(2));
	U3: bidir port map (lpt_oe(3), lpt_data(3), in_lpt_data(3), out_lpt_data(3));
	U4: bidir port map (lpt_oe(4), lpt_data(4), in_lpt_data(4), out_lpt_data(4));
	U5: bidir port map (lpt_oe(5), lpt_data(5), in_lpt_data(5), out_lpt_data(5));
	U6: bidir port map (lpt_oe(6), lpt_data(6), in_lpt_data(6), out_lpt_data(6));
	U7: bidir port map (lpt_oe(7), lpt_data(7), in_lpt_data(7), out_lpt_data(7));
end MAIN;

-- --------------------------------------------------
--  Port Description
-- --------------------------------------------------
library IEEE;
use IEEE.std_logic_1164.all;

-- component bidir
-- in-out port description for lpt-connector
entity bidir is
	port (
			oe         : in std_logic;
			ext_port   : inout std_logic;
			in_data    : in std_logic;
			out_data   : out std_logic
		);
end bidir;
architecture bi of bidir is
begin
	process (oe, in_data)
	begin
	case oe is
		when '1'    => ext_port <=  in_data;
		when '0'    => ext_port <= 'Z';
		when others => ext_port <= 'X';
		end case;
	end process;
	out_data <= ext_port;
end bi;

